#ifndef COUPLING_BOUNDARY_CONDITION_DESCRIPTOR_HPP
#define COUPLING_BOUNDARY_CONDITION_DESCRIPTOR_HPP

#include <mesh/IBoundaryDescriptor.hpp>
#include <scheme/IBoundaryConditionDescriptor.hpp>

#include <memory>

class CouplingBoundaryConditionDescriptor : public IBoundaryConditionDescriptor
{
 private:
  std::ostream&
  _write(std::ostream& os) const final
  {
    os << "coupling(" << *m_boundary_descriptor << ")";
    return os;
  }

  std::shared_ptr<const IBoundaryDescriptor> m_boundary_descriptor;
  size_t m_label;

 public:
  
  const IBoundaryDescriptor&
  boundaryDescriptor() const final
  {
    return *m_boundary_descriptor;
  }

  Type
  type() const final
  {
    return Type::coupling;
  }

  size_t
  label() const{
    return m_label;
  }

  CouplingBoundaryConditionDescriptor(std::shared_ptr<const IBoundaryDescriptor> boundary_descriptor, size_t label)
    : m_boundary_descriptor(boundary_descriptor), m_label{label}
  {
    ;
  }

  CouplingBoundaryConditionDescriptor(const CouplingBoundaryConditionDescriptor&) = delete;
  CouplingBoundaryConditionDescriptor(CouplingBoundaryConditionDescriptor&&)      = delete;

  ~CouplingBoundaryConditionDescriptor() = default;
};

#endif   // COUPLING_BOUNDARY_CONDITION_DESCRIPTOR_HPP
